﻿<?php

/*
Wiki-based text parser. 
(c)2016 by Bashkirov Aleksandr http://www.alib.spb.ru

Usage:

require "wiki-light-parser.php";
$text = wiki_parse($text);

Support format (based on dokuwiki):
==================================
**********************************

===== Text =====	=> <h1>Text</h1>			head 1
==== Text ====	=> 	=> <h2>Text</h2>			head 2
=== Text ===	=> 	=> <h3>Text</h3>			head 3

%% **No format1!** %%	=> Not formatting text			Don't parse text between %% %%
  * aaa			=> <ul><li>aaa</li>			Bullet list 
  * bbb			   <li>bbb</li></ul>

  - aaa			=> <ol><li>aaa</li>			Numeric list
  - bbb			   <li>bbb</li></ol>

-----			=> <hr>					Horisontal line

__test-text test-text__ => <i>test-text test-text</i>		Italic font
**test-text test-text** => <b>test-text test-text</b>		Bold font

[[http://alib.spb.ru]]	=> <a href='http://alib.spb.ru'>http://alib.spb.ru</a>	Hyperlink
[[http://alib.spb.ru/|txt]] => <a href='http://alib.spb.ru'>txt</a> Hyperlink with name

{{image.png}}		=> <img src='image.png'>		Image
{{image.png?50}}	=> <img width='50px' src='image.png'>	Image with "width"
{{image.png?50x20}}	=> <img width='50px' height="20px" src='image.png'> Image with "width" and height
{{image.png?50x20|Image hint}} => <img alt="Image hint" width='50px' height="20px" src='image.png'> Image with "width" and height? and with alternative text


*/

function wiki_l_sp($text='', $ul_pattern='  *', $ul_o='<ul>', $ul_z='</ul>')
{
$tmp = explode("\n", $text);
//print_r($tmp);
$out = '';
$first_ul = false;
foreach ($tmp as $key => $value)
	{
	if (substr($value,0,3)==$ul_pattern)
		{
		if ($first_ul==false) {$out.=$ul_o."\n"; $first_ul=true;}
		$out.="<li>".substr(trim($value),1)."</li>\n";
		}
	elseif ($first_ul==true) 
		{$out.=$ul_z."\n"; $first_ul=false;}
	else {$out.=rtrim($value)."\n";}
	}

return $out;
}

function wiki_l_href($inp)
{
$a=strpos($inp[1],'|');
if ($a===false)
	{$out = "<a target='_blank' href='".$inp[1]."'>".$inp[1]."</a>";}
else
	{$out = "<a target='_blank' href='".substr($inp[1],0,$a)."'>".substr($inp[1],$a+1)."</a>";}
return $out;
}

function wiki_l_img($inp)
{
$x=substr($inp[1],0,1);
$align='';
$img=$inp[1];
if ($x=='<') {$align='align="left"' ; $img=substr($inp[1],1);}
if ($x=='>') {$align='align="right"'; $img=substr($inp[1],1);}
if ($x=='*') {$align='align="center"'; $img=substr($inp[1],1);}
$a=strpos($img,'|');
if ($a===false)
	{$hint='';}
else
	{$hint=substr($img,$a+1); $img=substr($img,0,$a);}
$b=strpos($img,'?');
if ($b===false)
	{$w=''; $h='';}
	else
	{
	$hw=substr($img,$b+1);
	$img=substr($img,0,$b);
	$c=strpos($hw,'x');
	if ($c===false)
		{$h=''; $w="width='".$hw."px'";}
	else
		{$h="height='".substr($hw,$c+1)."px'";
		$w="width='".substr($hw,0,$c)."px'";
		}
	}
$out='<img '.$align.' alt="'.$hint.'" '.$w.' '.$h.' src="'.$img.'">';
$out=str_replace('   ',' ',$out);
$out=str_replace('  ',' ',$out);
return $out;
}

function wiki_l_nowiki($inp)
{ global $wiki_l_save, $wiki_l_save_idx;
$wiki_l_save[$wiki_l_save_idx]=$inp[1];
$ret = '%%~~~~~'.$wiki_l_save_idx.'~~~~~%%';
$wiki_l_save_idx++;
return $ret;
}

$wiki_l_save = array();
$wiki_l_save_idx = 0;

function wiki_parse($text1)
{ global $wiki_l_save, $wiki_l_save_idx;
$text = trim($text1);
$text = preg_replace_callback("!%%(.*?)%%!si","wiki_l_nowiki",$text);
$text = preg_replace("!=====(.*?)=====!si","<h1>\\1</h1>",$text);
$text = preg_replace("!====(.*?)====!si","<h2>\\1</h2>",$text);
$text = preg_replace("!===(.*?)===!si","<h3>\\1</h3>",$text);
//$text = preg_replace("!==(.*?)==!si","<h4>\\1</h4>",$text);
//$text = preg_replace("!=(.*?)=!si","<h5>\\1</h5>",$text);
$text = preg_replace("!__(.*?)__!si","<i>\\1</i>",$text);
$text = preg_replace("!\*\*(.*?)\*\*!si","<b>\\1</b>",$text);
$text = preg_replace_callback("!\[\[(.*?)\]\]!si","wiki_l_href",$text);
$text = preg_replace_callback("!{{(.*?)}}!si","wiki_l_img",$text);
$text = wiki_l_sp($text, '  -', '<ol>', '</ol>');
$text = wiki_l_sp($text, '  *', '<ul>', '</ul>');
$text = str_replace('-----','<hr>',$text);

$tmp = explode("\n", $text);
$i=0;
foreach ($tmp as $key => $value)
	{if (substr(trim($value),-1)!='>') {$tmp[$i]=$value."<br>";} $i++;}
$text = implode("\n", $tmp);

foreach ($wiki_l_save as $key => $value)
	{
	$text = str_replace('%%~~~~~'.$key.'~~~~~%%', $value, $text);
	}
return $text;
}

?>